package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.constant.CommonAttributes;
import cn.wisenergy.common.utils.*;
import cn.wisenergy.mapper.CardMapper;
import cn.wisenergy.mapper.RefillCardMapper;
import cn.wisenergy.model.app.CardInfo;
import cn.wisenergy.model.app.RefillCard;
import cn.wisenergy.model.dto.RefillCardDto;
import cn.wisenergy.model.vo.UserQueryVo;
import cn.wisenergy.service.app.RefillCardService;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

@Service
@Slf4j
public class RefillCardServiceImpl implements RefillCardService {
    @Autowired
    RefillCardMapper refillCardMapper;

    @Autowired
    private CardMapper cardMapper;


    @Override
    @Transactional
    public R<Boolean> add(RefillCard refillCard) {
        log.info("RefillCardServiceImpl[].add[].input.param:refillCard:{}"+refillCard);

        if (refillCard.getCardNumber()==0||null==refillCard.getCardNumber()) {
            return R.error("输入的卡数无效是无效值");
        }
        //设置初始状态
        String time = DateUtil.getTime(new Date(), "yyyy.MM.dd.HH.mm.sss");
        refillCard.setBatchNumber(time);
        refillCard.setIsActivite(1);
        refillCard.setIsMakeCard(1);
        refillCard.setIsDelete(0);
        int add = refillCardMapper.add(refillCard);

        //判断数据是否添加成功
        if (add==0) {
            return R.error("数据添加失败");
        }
        return R.ok(0,true);
    }

    @Override
    public R<Boolean> update(String batchNumber) {
        log.info("RefillCardServiceImpl[].update[].input.param:batchNumber:"+batchNumber);
        if (StringUtils.isEmpty(batchNumber)) {
            return R.error("传入参数为空");
        }
        int update = refillCardMapper.update(batchNumber);
        if (update==0) {
            return R.error("数据修改失败");
        }
        return R.ok(0,true);
    }

    @Override
    public R<PageInfo<RefillCardDto>> getList(Integer pageNum, Integer pageSize) {
        log.info("RefillCardServiceImpl[].getList[].input.param:pageNum,pageSize"+pageNum,pageSize);
        if (null == pageSize || pageSize == 0) {
            pageSize = 10;
        }
        if (null == pageNum || pageNum == 0) {
            pageNum = 1;
        }

        //创建参数容器
        HashMap<String, Object> map = new HashMap<>(4);
        map.put("pageNo",pageNum-1);
        map.put("pageSize",pageSize);
        ArrayList<RefillCardDto> refillCardDtos = new ArrayList<>();

        //查询参数
        List<RefillCard> list = refillCardMapper.getList(map);
        for (RefillCard refillCard : list) {
            RefillCardDto refillCardDto = new RefillCardDto();
            BeanUtils.copyProperties(refillCard,refillCardDto);
            refillCardDtos.add(refillCardDto);
        }

        PageInfo<RefillCardDto> refillCardDtoPageInfo = new PageInfo<>();
        refillCardDtoPageInfo.setTotal(refillCardMapper.getUserNumbers());
        refillCardDtoPageInfo.setPageNum(pageNum);
        refillCardDtoPageInfo.setPageSize(pageSize);
        refillCardDtoPageInfo.setList(refillCardDtos);
        return R.ok(refillCardDtoPageInfo);
    }

    @Override
    @Transactional
    public R<Boolean> createCard(Integer id) {
        log.info("RefillCardServiceImpl[].createCard[].input.param:id"+id);
        if (null==id||id==0) {
            return R.error("传入参数为空");
        }

        //查询充值卡信息
        RefillCard byId = refillCardMapper.getById(id);
        if (null==byId) {
            return R.error("该充值卡不存在");
        }

        //获取充值卡的卡数，并生成卡片
        ArrayList<CardInfo> cardInfos = new ArrayList<>();

        //子卡添加
        for (int i = 0; i < byId.getCardNumber(); i++) {
            CardInfo cardInfo = new CardInfo();
            cardInfo.setBatchNumber(byId.getBatchNumber());
            cardInfo.setIsDelete(0);
            cardInfo.setMoney(498);
            cardInfo.setStatus(0);
            cardInfo.setLimit(3);
            cardInfo.setSecretKey(SecretkeyUtil.getSecretkey());
            cardInfo.setCardNumber(CardNumberUtil.cardNumber());
            cardInfos.add(cardInfo);
        }
        int add1 = cardMapper.add(cardInfos);

        if (add1==0) {
            R.error("数据添加失败");
        }
        int isMakeCard = refillCardMapper.setIsMakeCard(id);

        if (isMakeCard==0) {
            R.error("充值卡是否制卡状态修改失败");
        }
        return R.ok(0,true);
    }

}
