package cn.wisenergy.service.app.impl;

import cn.wisenergy.common.expection.BaseException;
import cn.wisenergy.common.utils.R;
import cn.wisenergy.mapper.SchemeMapper;
import cn.wisenergy.mapper.VolunteerMapper;
import cn.wisenergy.model.app.SchemeInfo;
import cn.wisenergy.model.app.Volunteer;
import cn.wisenergy.service.app.VolunteerService;
import cn.wisenergy.service.common.VolunteerListener;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Date;
import java.util.List;

/**
 * @ Description: 志愿接口实现
 * @ Author     : 86187
 * @ Date       : 2021/1/8 9:44
 */
@Service
@Slf4j
public class VolunteerServiceImpl extends ServiceImpl<VolunteerMapper, Volunteer> implements VolunteerService {
    @Autowired
    private VolunteerMapper volunteerMapper;

    @Autowired
    private SchemeMapper schemeMapper;

    @Autowired
    private VolunteerService volunteerService;

    @Override
    public R<Boolean> batchAdd(List<Volunteer> list) {
        return null;
    }

    @Override
    public void excelAdd(MultipartFile file, Integer type, String schemeName, HttpServletResponse response) throws IOException {
        if (file == null || file.isEmpty() || null == type || StringUtils.isBlank(schemeName)) {
            throw new BaseException("操作错误");
        }

        //判断文件类型=
        boolean valid = this.validContentType(file.getOriginalFilename());
        if (!valid) {
            throw new BaseException("请传入Excel文件");
        }
        VolunteerListener excelListener = new VolunteerListener();

        //2.读取数据进行入库操作
        EasyExcel.read(file.getInputStream(), Volunteer.class, new VolunteerListener()).sheet().doRead();
        List<Volunteer> list = excelListener.getList();

        //3、保存方案和志愿信息
        SchemeInfo schemeInfo = new SchemeInfo();
        schemeInfo.setSchemeName(schemeName);
        schemeInfo.setType(type);
        schemeInfo.setUploadTime(new Date());
        schemeInfo.setIsDelete(0);

        saveSchemeAndVolunteer(schemeInfo, list);
    }

    @Override
    public void ceshi(MultipartFile file, HttpServletResponse response) throws IOException {
        if (file == null || file.isEmpty()) {
            throw new BaseException("操作错误");
        }

        VolunteerListener excelListener = new VolunteerListener();

        //判断文件类型=
        boolean valid = this.validContentType(file.getOriginalFilename());
        if (!valid) {
            throw new BaseException("请传入Excel文件");
        }
        //2.读取数据进行入库操作
        EasyExcel.read(file.getInputStream(), Volunteer.class, excelListener).sheet().doRead();
        List<Volunteer> list = excelListener.getList();

        //3、保存方案和志愿信息
        SchemeInfo schemeInfo = new SchemeInfo();
        schemeInfo.setSchemeName("专科文学编导一批");
        schemeInfo.setType(7);
        schemeInfo.setUploadTime(new Date());
        schemeInfo.setIsDelete(0);

        saveSchemeAndVolunteer(schemeInfo, list);
        log.info("导入方案数据成功!");
    }

    @Override
    public R<SchemeInfo> add(SchemeInfo schemeInfo) {
        int count = schemeMapper.add(schemeInfo);
        log.info("返回对象：" + schemeInfo);

        int id = schemeInfo.getId();
        log.info("主键id:" + id);
        return R.ok(schemeInfo);
    }

    /**
     * 保存方案和志愿信息
     *
     * @param schemeInfo 方案信息
     * @param list       志愿信息
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveSchemeAndVolunteer(SchemeInfo schemeInfo, List<Volunteer> list) {
        //1、保存方案信息
        int count = schemeMapper.add(schemeInfo);
        if (count == 0) {
            throw new RuntimeException("保存方案信息失败!");
        }

        //保存志愿信息
        for (Volunteer volunteer : list) {
            volunteer.setSchemeId(schemeInfo.getId());
            volunteer.setType(schemeInfo.getType());
        }
        boolean bool = volunteerService.saveBatch(list);
        if (!bool) {
            throw new RuntimeException("保存志愿信息失败!");
        }
    }

    /**
     * 判断文件类型
     *
     * @param fileName 文件名
     * @return true or false
     */
    public boolean validContentType(String fileName) {
        if (StringUtils.isBlank(fileName)) {
            return false;
        }
        String substring = fileName.substring(fileName.lastIndexOf(".") + 1);
        if (substring.equals("xlsx") || substring.equals("xls")) {
            return true;
        }
        return false;
    }
}
