package cn.wisenergy.web.admin.controller.app;

import cn.wisenergy.common.utils.R;
import cn.wisenergy.model.app.User;
import cn.wisenergy.model.dto.UserCommitDto;
import cn.wisenergy.model.dto.UserInfoDto;
import cn.wisenergy.model.vo.UserInfoVo;
import cn.wisenergy.model.vo.UserQueryVo;
import cn.wisenergy.model.vo.UserShowVo;
import cn.wisenergy.service.app.UserService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * @ Description: 用户管理模
 * @ Author     : 86187
 * @ Date       : 2021/1/7 10:29
 * @author 86187
 */
@RestController
@Api(tags = "PC-用户管理")
@RequestMapping("/user")
@Slf4j
public class UserController {
    @Autowired
    private UserService userService;


    @ApiOperation(value = "用户管理", notes = "用户管理", httpMethod = "POST")
    @ApiImplicitParam(name = "queryVo", value = "用户信息", dataType = "UserQueryVo")
    @PostMapping("/manage")
    public R<PageInfo<UserInfoDto>> manageUser(@RequestBody UserQueryVo queryVo) {
        log.info("UserController[]manageUser[]input.param:queryV0:" + queryVo);
        if (null == queryVo || null == queryVo.getPageSize() || null == queryVo.getPageNo()) {
            return R.error("入参为空!");
        }
        //返回数据
        return userService.getUserList(queryVo);
    }

    @ApiOperation(value = "提交个人信息", notes = "提交个人信息", httpMethod = "POST")
    @ApiImplicitParam(name = "userInfo", value = "个人信息", dataType = "UserCommitDto")
    @PostMapping("/commit")
    public R<UserInfoVo> commitUserInfo(@RequestBody UserCommitDto userInfo) {
        log.info("UserController[]commitUserInfo[]input.param:queryV0:" + userInfo);
        if (null == userInfo) {
            return R.error("入参为空!");
        }
        //返回数据
        return userService.commitUserInfo(userInfo);
    }

    @ApiOperation(value = "编辑个人信息", notes = "编辑个人信息", httpMethod = "POST")
    @ApiImplicitParam(name = "userInfo", value = "个人信息", dataType = "UserCommitDto")
    @PostMapping("/edit")
    public R<Boolean> edit(@RequestBody UserCommitDto userInfo) {
        log.info("UserController[]edit[]input.param:queryV0:" + userInfo);
        if (null == userInfo) {
            return R.error("入参为空!");
        }
        //返回数据
        return userService.edit(userInfo);
    }

    @ApiOperation(value = "获取个人信息", notes = "获取个人信息", httpMethod = "GET")
    @ApiImplicitParam(name = "userId", value = "用户id", dataType = "int")
    @GetMapping("/getById")
    public R<UserShowVo> getById(Integer userId) {
        log.info("UserController[]getById[]input.param:queryV0:" + userId);
        if (null == userId) {
            return R.error("入参为空!");
        }

        //返回数据
        return userService.getById(userId);
    }

    @ApiOperation(value = "测试like循环查询", notes = "测试like循环查询", httpMethod = "GET")
    @GetMapping("/test")
    public R<List<User>> test() {
        log.info("UserController[]test[]input.param");

        //返回数据
        return userService.test();
    }

}
